"""Version of the module"""

import re

VERSION_OUTPUT_RE = r".*?(\d+\.\d+\.\d+\.\d+).*"


def static_vars(**kwargs):
    def decorate(func):
        for k in kwargs:
            setattr(func, k, kwargs[k])
        return func
    return decorate


@static_vars(prog=re.compile(VERSION_OUTPUT_RE, re.MULTILINE))
def extract_version(input_str):
    result = None
    for match in extract_version.prog.finditer(input_str):
        result = match.group(1)
    return result



"""Usefull tools to set the enviroment"""
import os
import tempfile
from functools import lru_cache

import cpuinfo
from logzero import logger

from . import download_driver


@lru_cache(maxsize=1)
def get_cpu_arch():
    """Try to guess what cpu architecture we are running on"""
    manufacturer = cpuinfo.get_cpu_info().get('brand_raw')
    arch = 'arm' if 'm1' in manufacturer.lower() else 'x86_64'
    return arch


def is_fs_case_sensitive():
    """Is the filesystem case sensitive"""
    if not hasattr(is_fs_case_sensitive, 'case_sensitive'):
        with tempfile.NamedTemporaryFile(prefix='TmP') as tmp_file:
            setattr(is_fs_case_sensitive,
                    'case_sensitive',
                    not os.path.exists(tmp_file.name.casefold()))
    return is_fs_case_sensitive.case_sensitive


def _clean_and_add_env_path(add_path):
    # Clean the already defined path
    # Remove multiple identincal entries
    cleaned_path = []
    abs_path = [os.path.abspath(x)
                for x in os.environ['PATH'].split(os.pathsep)]

    for c_path in abs_path:
        if c_path:
            if is_fs_case_sensitive():
                case_path = c_path
                checked_paths = cleaned_path
            else:
                case_path = c_path.casefold()
                checked_paths = [x.casefold() for x in cleaned_path]

            if case_path not in checked_paths:
                cleaned_path.append(c_path)

    # Add the new path in the start of the enviroment $PATH
    if add_path:
        add_path = os.path.abspath(add_path)
        if is_fs_case_sensitive():
            add_path_case = add_path
        else:
            add_path_case = add_path.casefold()
        for c_path in cleaned_path[:]:
            if is_fs_case_sensitive():
                case_path = c_path
            else:
                case_path = c_path.casefold()

            if add_path_case == case_path:
                cleaned_path.remove(c_path)

        cleaned_path.insert(0, add_path)

    os.environ['PATH'] = os.pathsep.join(cleaned_path)


def safely_set_chromedriver_path(**kwargs):
    """Manipulate the enviroment PATH to include the chromedriver folder"""
    chromedriver_path = download_driver.download_only_if_needed(chrome_path=kwargs.get(
        'chrome_path'), chromedriver_folder=kwargs.get('chromedriver_folder'))
    if chromedriver_path:
        logger.debug("Adding %s to PATH", chromedriver_path)
        _clean_and_add_env_path(chromedriver_path)



"""A Module to search and find the chrome executable."""

import os
import platform
import shutil
import subprocess
from functools import lru_cache

import ubelt as ub
from logzero import logger

from . import version
from . import enviroment


@lru_cache(maxsize=1)
def _get_chrome_executable():
    system_name = platform.system()
    if system_name == 'Windows':
        return "chrome.exe"
    if system_name == 'Linux':
        return "google-chrome"
    if system_name == 'Darwin':
        return "Google Chrome"
    return None


def _windows_program_locations():
    possible_folders = [os.environ.get(
        i) for i in ["ProgramFiles(x86)", "ProgramW6432"]]
    possible_folders.append(str(ub.Path.home()))
    for folder in possible_folders:
        if folder:
            logger.debug("Searching in: %s", folder)
            yield folder


def _darwin_applications():
    possible_folders = ['/Applications', '/']
    for folder in possible_folders:
        if folder:
            logger.debug("Searching in: %s", folder)
            yield folder


def _is_exe(fpath):
    return os.path.exists(fpath) and os.access(fpath, os.X_OK) and os.path.isfile(fpath)


def _find_chrome_in_subfolders(folder):
    if enviroment.is_fs_case_sensitive():
        search_filename = _get_chrome_executable().strip()
    else:
        search_filename = _get_chrome_executable().casefold().strip()

    for root, _dirs, files in os.walk(folder):
        for filename in files:
            if enviroment.is_fs_case_sensitive():
                case_filename = filename
            else:
                case_filename = filename.casefold()
            if case_filename == search_filename:
                filepath = os.path.join(root, filename)
                if _is_exe(filepath):
                    return filepath
    return None


@lru_cache(maxsize=1)
def get_path():
    """Get the path of a Google Chrome installation."""
    logger.debug("Searching for Google Chrome installations...")
    system_name = platform.system()

    if enviroment.is_fs_case_sensitive():
        search_filename = _get_chrome_executable().strip()
    else:
        search_filename = _get_chrome_executable().casefold().strip()
    in_path = shutil.which(search_filename)
    if in_path:
        return in_path

    if system_name == 'Windows':
        for folder in _windows_program_locations():
            filepath = _find_chrome_in_subfolders(folder)
            if filepath:
                return filepath

    if system_name == 'Darwin':
        for folder in _darwin_applications():
            filepath = _find_chrome_in_subfolders(folder)
            if filepath:
                return filepath

    logger.error("Google Chrome wasn't found in the usual locations")
    return None


@lru_cache(maxsize=None)
def get_version(chrome_path=None):
    """Get the version of a Google Chrome Executable"""
    system_name = platform.system()
    if chrome_path:
        if not _is_exe(chrome_path):
            logger.error(
                "%s is not a valid Google Chrome executable.", chrome_path)
            return None
    else:
        chrome_path = get_path()

    version_str = None
    if chrome_path:
        if system_name == 'Windows':
            output = subprocess.check_output(
                'powershell -command "&{(Get-Item \'%s\').VersionInfo.ProductVersion}"' % (
                    chrome_path),
                shell=True)
        else:
            output = subprocess.check_output(
                f'"{chrome_path}" --version', shell=True)
        output_str = output.decode(encoding='ascii')
        version_str = version.extract_version(output_str)
        logger.debug("Google Chrome Version: %s", version_str)
    return version_str


@lru_cache(maxsize=None)
def get_architecture(chrome_path=None):
    """Get the arcitecture (32bit or 63bit) of the Google Chrome executable."""
    if chrome_path:
        if not _is_exe(chrome_path):
            logger.error(
                "%s is not a valid Google Chrome executable.", chrome_path)
            return None
    else:
        chrome_path = get_path()

    (bits, _linkage) = platform.architecture(executable=chrome_path)
    logger.debug("Google Chrome is a %s executable", bits)
    return bits
